//*************************************************************************************
//
//  SoR Scope - software to provide Oscilloscope-like with cheap Microcontrollers
//  and free PC software
//
//  Copyright (C) 2009,	Cosmin Prund
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//*************************************************************************************


#include "prepconst.h"
#include "cmdhandle.h"
#include "fastusart.h"
#include "customize.h"
#include "scopeadc.h"

// Status for the state machine
unsigned char c_state = 0;
volatile int ScopeOutputMode;

unsigned char temp1;
unsigned char temp2;
unsigned char temp3;

// Show AT command prompt
void show_AT_commad_prompt()
{
	SendOutString(">");
}

// Plain AT command
void handle_AT()
{
	SendOutString("OK: ");
	SendOutString(MCUID);
	SendOutString("\n\r");
	if (ScopeOutputMode == 1) show_AT_commad_prompt();	
}


// AT command ERROR
void handle_AT_ERROR()
{
	SendOutString("ERROR: Unkown AT command, try AT? for information on available commands\n\r");
	if (ScopeOutputMode == 1) show_AT_commad_prompt();	
}

// AT command ?
void handle_AT_HELP()
{	
	SendOutString("You can use the following commands on the ");
	SendOutString(MCUID);
	SendOutString(":\n\r");
	SendOutString("AT?  -   Prints THIS help message\n\r");
	SendOutString("ATI  -   Prints information about the capabilities of\n\r");
	SendOutString("         this SoR Scope Software\n\r");
	SendOutString("ATP  -   Enter Interactive Command Prompt Mode. ATP is the\n\r");
	SendOutString("         default mode of operation for the SoR Scope and it\n\r");
	SendOutString("         allows you to enter commands one by one from a\n\r");
	SendOutString("         terminal application\n\r");
	SendOutString("ATDUMP - This is THE most important command for the SoR Scope!\n\r");
	SendOutString("         Here's the full sintax:\n\r");
	SendOutString("ATD[UMP] (B[INARY]|D[ECIMAL]|H[EX]) (A[LL]|P#) (4[BIT]|8[BIT]|1[0BIT])\n\r");
	SendOutString("The stuff in square brackets is optional. AKA: you need to provide\n\r");
	SendOutString("of the command for the processor to understand what you're talking\n\r");
	SendOutString("about. BINARY/DECIMAL/HEX specifies the output format. Don't use\n\r");
	SendOutString("BINARY if you want to see the result in a terminal application!\n\r");
	SendOutString("Only the first letter of BINARY/DECIMAL/HEX is required.\n\r");
	SendOutString("The following parameter is \"ALL\" or the pin number. The number\n\r");
	SendOutString("The PIN number is actually the index of the PIN as configured at\n\r");
	SendOutString("application compile time! To see the list of pins issue\n\r");
	SendOutString("the ATI command at the prompt.\n\r");
	SendOutString("the 4BIT/8BIT/10BIT gives the sampling mode.\n\r");
	SendOutString("Examples:\n\r");
	SendOutString("  ATDUMP DECIMAL ALL 8BIT\n\r");
	SendOutString("    - Dumps all configured PINS using DECIMAL notation, 8bit samples\n\r");
	SendOutString("  ATD H P1 4BIT\n\r");
	SendOutString("    - Dumps Pin1 (the second pin) using HEX notation, 4bit samples\n\r");
	SendOutString("-----------------\n\r\n\r");
	if (ScopeOutputMode == 1) show_AT_commad_prompt();	
}

// AT command "I"
void handle_AT_INFO()
{
	SendOutString("This is the SoR Scope project. For more details visit http://www.societyofrobots.com and ask in the forums\n\r");
	SendOutString("The software is provided under the terms of the GNU General Public License version 3 or later\n\r");
	SendOutString("You can get a copy of the licence at http://www.gnu.org/licenses\n\r");
	SendOutString("Following is a list of name-value pairs providing a list of details and capabilities for this instance of the scope\n\r");
	SendOutString("--------------------------------------\n\r");
	SendOutString("MCUID="); SendOutString(MCUID); SendOutString("\n\r");
	SendOutString("MCUNAME="); SendOutString(MCUNAME); SendOutString("\n\r");
	SendOutString("VERSION=1\n\r");
	for (int i=0; i < AvailableAdcCount;i++)
	{
		SendOutString("PIN");
		FastPrintDecimalNumber(i);
		SendOutString(": ");
		SendOutString(AdcNames[i]);
		SendOutString("\n\r");
	};
	if (ScopeOutputMode == 1) show_AT_commad_prompt();	
}

// Change Command Mode for ATDUMP command
void ChangeCommandModeForATDUMP()
{
	SendOutString("ATDUMP ");
	if (temp1 == 'D')
		{
			SendOutString("DECIMAL ");
			if (temp2 == 'A')
				{
					SendOutString("ALL ");
					FastPrintDecimalNumber(temp3);
					SendOutString("Bit\n\r");
					// DECIMAL ALL
					if (temp3 == 8) ScopeOutputMode = 2;
					else SendOutString("NOT implemented!\n\r");
				}
			else if ((temp2 >= 0) && (temp2 < MAX_NUM_ADC))
				{
					SendOutString("Port");
					FastPrintDecimalNumber(temp2);
					SendOutString(" ");
					FastPrintDecimalNumber(temp3);
					SendOutString("Bit\n\r");
					// DECIMAL SINGLE PORT
					SendOutString("NOT implemented!\n\r");
				}
			else
				{
					SendOutString("Errror\n\r");
				}
		}
	else if (temp1 == 'H')
		{
			SendOutString("HEX ");
			if (temp2 == 'A')
				{
					SendOutString("ALL ");
					FastPrintDecimalNumber(temp3);
					SendOutString("Bit\n\r");
					// HEX ALL					
					SendOutString("NOT implemented!\n\r");
				}
			else if ((temp2 >= 0) && (temp2 < MAX_NUM_ADC))
				{
					SendOutString("Port");
					FastPrintDecimalNumber(temp2);
					SendOutString(" ");
					FastPrintDecimalNumber(temp3);
					SendOutString("Bit\n\r");
					// HEX SINGLE PORT
					SendOutString("NOT implemented!\n\r");
				}
			else
				{
					SendOutString("Errror\n\r");
				}
		}
	else if (temp1 == 'B')
		{
			SendOutString("BINARY ");	
			if (temp2 == 'A')
				{
					SendOutString("ALL ");
					FastPrintDecimalNumber(temp3);
					SendOutString("Bit\n\r");
					// BINARY ALL					
					SendOutString("NOT implemented!\n\r");
				}
			else if ((temp2 >= 0) && (temp2 < MAX_NUM_ADC))
				{
					SendOutString("Port");
					FastPrintDecimalNumber(temp2);
					SendOutString(" ");
					FastPrintDecimalNumber(temp3);
					SendOutString("Bit\n\r");
					// BINARY SINGLE PORT
					SendOutString("NOT implemented!\n\r");
				}
			else
				{
					SendOutString("Errror\n\r");
				}
		}
	if (ScopeOutputMode == 1) show_AT_commad_prompt();	
}

// AT command handling state machine
void cmd_handle_char(char c)
{
	if ((c == 13) | (c == 10)) 
		{
			if (ScopeOutputMode == 1) SendOutString("\n\r");
			c = 13;
		}
	else if ((c!=10) && (c!= 8) && (c!=0x7F))
		{
			// Trun chars in uppercase chars
			if ((c >= 0x61) && (c <= 0x7A))
				c -= 0x20;
			if (ScopeOutputMode == 1) USART_SEND_BYTE(c);
		}
	else
		c = 0;

	if (c != 0)
	{

		//SendOutString("(");
		//FastPrintDecimalNumber(c_state);
		//SendOutString("->");	


		switch(c_state)	{
			case 0: // <START>
				if (c == 'A') c_state = 1;
				else if ((c == 13) && (ScopeOutputMode == 1)) handle_AT_ERROR();
				break;
			case 1: // <START> "A"
				if (c == 'T') c_state = 2;
				else c_state = 0;
				break;
			case 2: // <START> "A" "T"
				if (c == 13)
					{
						handle_AT();
						c_state = 0;
					}
				else if (c == 'I') c_state = 4;
				else if (c == 'P') c_state = 6;
				else if (c == '?') c_state = 5;
				else if (c == 'D') c_state = 7;
				else c_state = 3;
				break;
			case 3: // <START> "A" "T" -=ERROR=-
				if (c == 13)
				{
					c_state = 0;
					handle_AT_ERROR();
				}
				break;
			case 4: // <START> "A" "T" "I"
				if (c == 13)
					{
						handle_AT_INFO();
						c_state = 0;
					}
				else
					c_state = 3; // Error mode
				break;
			case 5: // <START> <A> <T> <?>
				if (c == 13)
					{
						handle_AT_HELP();
						c_state = 0;
					}
				else c_state =3; // Enter error mode					
				break;
			case 6: // <START> <A> <T> <P>
				if (c == 13)
					{
						c_state = 0;
						ScopeOutputMode = 1;
						SendOutString("\n\rInteractive Prompt Mode\n\r>");
					}
				else c_state = 3; // Enter error mode
				break;
			case 7: // <START> <A> <T> <D>
				if (c == ' ') c_state = 11;
				else if (c == 'U') c_state = 8;
				else if (c == 13)
					{
						c_state = 0;
						handle_AT_ERROR();
					}
				else c_state = 3; // enter error mode
				break;
			case 8: // <START> <A> <T> <D> <U>
				if (c == ' ') c_state = 11;
				else if (c == 'M') c_state = 9;
				else if (c == 13)
					{
						c_state = 0;
						handle_AT_ERROR();
					}
				else c_state = 3; // enter error mode
				break;
			case 9: // <START> <A> <T> <D> <U> <M>
				if (c == ' ') c_state = 11;
				else if (c == 'P') c_state = 10;
				else if (c == 13)
					{
						c_state = 0;
						handle_AT_ERROR();
					}
				else c_state = 3; // enter error mode
				break;
			case 10:
				if (c == ' ') c_state = 11;
				else if (c == 13)
					{
						c_state = 0;
						handle_AT_ERROR();
					}
				else c_state = 3; // enter error mode
				break;
			case 11: // <START> <ATDUMP> < >
				if (c == 13)
					{
						c_state = 0;
						handle_AT_ERROR();
					}
				else if (c != ' ') // ignore space; allow multiple spaces
				{
					if (c == 'B') {temp1='B'; c_state = 12;}
					else if (c == 'H') {temp1='H'; c_state = 18;}
					else if (c == 'D') {temp1='D'; c_state = 21;}
					else c_state = 3; // enter error mode
				}
				break;
			case 12: // <START> <ATDUMP> <B>
				if (c == ' ') c_state = 28;
				else if (c == 13)
					{
						c_state = 0;
						handle_AT_ERROR();
					}
				else if (c == 'I') c_state = 13;
				else c_state = 3;
				break;
			case 13: // <START> <ATDUMP> <BI>
				if (c == ' ') c_state = 28;
				else if (c == 13)
					{
						c_state = 0;
						handle_AT_ERROR();
					}
				else if (c == 'N') c_state = 14;
				else c_state = 3;
				break;
			case 14: // <START> <ATDUMP> <BIN>
				if (c == ' ') c_state = 28;
				else if (c == 13)
					{
						c_state = 0;
						handle_AT_ERROR();
					}
				else if (c == 'A') c_state = 15;
				else c_state = 3;
				break;
			case 15: // <START> <ATDUMP> <BINA>
				if (c == ' ') c_state = 28;
				else if (c == 13)
					{
						c_state = 0;
						handle_AT_ERROR();
					}
				else if (c == 'R') c_state = 16;
				else c_state = 3;
				break;
			case 16: // <START> <ATDUMP> <BINAR>
				if (c == ' ') c_state = 28;
				else if (c == 13)
					{
						c_state = 0;
						handle_AT_ERROR();
					}
				else if (c == 'Y') c_state = 17;
				else c_state = 3;
				break;
			case 17: // <START> <ATDUMP> <BINARY>
				if (c == ' ') c_state = 28;
				else if (c == 13)
					{
						c_state = 0;
						handle_AT_ERROR();
					}
				else c_state = 3; // error mode
				break;			
			case 18: // <START> <ATDUMP> <H>
				if (c == ' ') c_state = 28;
				else if (c == 13)
					{
						c_state = 0;
						handle_AT_ERROR();
					}
				else if (c == 'E') c_state = 19;
				else c_state = 3;
				break;
			case 19: // <START> <ATDUMP> <HE>
				if (c == ' ') c_state = 28;
				else if (c == 13)
					{
						c_state = 0;
						handle_AT_ERROR();
					}
				else if (c == 'X') c_state = 20;
				else c_state = 3;
				break;
			case 20: // <START> <ATDUMP> <HEX>
				if (c == ' ') c_state = 28;
				else if (c == 13) {c_state = 0; handle_AT_ERROR();}
				else c_state = 3;
				break;
			case 21: // <START> <ATDUMP> <D>
				if (c == ' ') c_state = 28;
				else if (c == 13) {c_state = 0; handle_AT_ERROR();}
				else if (c == 'E') c_state = 22;
				else c_state = 3;
				break;
			case 22: // <START> <ATDUMP> <D>
				if (c == ' ') c_state = 28;
				else if (c == 13) {c_state = 0; handle_AT_ERROR();}
				else if (c == 'C') c_state = 23;
				else c_state = 3;
				break;
			case 23: // <START> <ATDUMP> <D>
				if (c == ' ') c_state = 28;
				else if (c == 13) {c_state = 0; handle_AT_ERROR();}
				else if (c == 'I') c_state = 24;
				else c_state = 3;
				break;
			case 24: // <START> <ATDUMP> <D>
				if (c == ' ') c_state = 28;
				else if (c == 13) {c_state = 0; handle_AT_ERROR();}
				else if (c == 'M') c_state = 25;
				else c_state = 3;
				break;
			case 25: // <START> <ATDUMP> <D>
				if (c == ' ') c_state = 28;
				else if (c == 13) {c_state = 0; handle_AT_ERROR();}
				else if (c == 'A') c_state = 26;
				else c_state = 3;
				break;
			case 26: // <START> <ATDUMP> <D>
				if (c == ' ') c_state = 28;
				else if (c == 13) {c_state = 0; handle_AT_ERROR();}
				else if (c == 'L') c_state = 27;
				else c_state = 3;
				break;
			case 27: // <START> <ATDUMP> <D>
				if (c == ' ') c_state = 28;
				else if (c == 13) {c_state = 0; handle_AT_ERROR();}
				else c_state = 3;
				break;
			case 28: // <START> <ATDUMP> <BINARY|DECIMAL|HEX>
				if (c != ' ') // Ignore spaces
				{
					// So it's not an space; Let's see what we'v got!
					if (c == 13)
						{
							c_state = 0;
							handle_AT_ERROR();
						}
					else if (c == 'A') {temp2='A'; c_state = 29;}
					else if (c == 'P') {temp2=0; c_state=32;}
					else c_state = 3;
				};
				break;
			case 29:
				if (c == ' ') c_state = 33;
				else if (c == 'L') c_state = 30;
				else if (c == 13) {c_state = 0; handle_AT_ERROR();}
				else c_state = 3;
				break;
			case 30:
				if (c == ' ') c_state = 33;
				else if (c == 'L') c_state = 31;
				else if (c == 13) {c_state = 0; handle_AT_ERROR();}
				else c_state = 3;
				break;
			case 31:
				if (c == ' ') c_state = 33;
				else if (c == 13) {c_state = 0; handle_AT_ERROR();}
				else c_state = 3;
				break;
			case 32:
				if (c == ' ') c_state = 33;
				else if ((c >= '0') && (c <= '9')) temp2 = temp2 * 10 + (c & 0x0F);
				else if (c == 13) {c_state=0; handle_AT_ERROR();}
				else c_state = 3;
				break;
			case 33:
				if (c == '4') {temp3 = 4; c_state = 34;}
				else if (c == '8') {temp3 = 8; c_state = 34;}
				else if (c == '1') {temp3 = 10;c_state = 38;}
				else if (c == 13) {c_state = 0; handle_AT_ERROR();}
				else c_state = 3;
				break;
			case 34:
				if (c == 13)
					{
						// Done! I've got everything I need in temp1, temp2 and temp3
						c_state = 0;
						ChangeCommandModeForATDUMP();
					}
				else if (c == 'B') c_state = 35;
				else c_state = 3; // error mode
				break;
			case 35:
				if (c == 13)
					{
						// Done! I've got everything I need in temp1, temp2 and temp3
						c_state = 0;
						ChangeCommandModeForATDUMP();
					}
				else if (c == 'I') c_state = 36;
				else c_state = 3; // error mode
				break;
			case 36:
				if (c == 13)
					{
						// Done! I've got everything I need in temp1, temp2 and temp3
						c_state = 0;
						ChangeCommandModeForATDUMP();
					}
				else if (c == 'T') c_state = 37;
				else c_state = 3; // error mode
				break;
			case 37:
				if (c == 13)
					{
						// Done! I've got everything I need in temp1, temp2 and temp3
						c_state = 0;
						ChangeCommandModeForATDUMP();
					}
				else c_state = 3; // error mode
				break;
			case 38:
				if (c == 13)
					{
						// Done! I've got everything I need in temp1, temp2 and temp3
						c_state = 0;
						ChangeCommandModeForATDUMP();
					}
				else if (c == '0') c_state = 34;
				else c_state = 3; // error mode
				break;
			default:
				// Bug! The state machine failed (unknown state)
				c_state = 0; // start over
		}

		//FastPrintDecimalNumber(c_state);
		//SendOutString(")");	
	}
}
