//*************************************************************************************
//
//  SoR Scope - software to provide Oscilloscope-like with cheap Microcontrollers
//  and free PC software
//
//  Copyright (C) 2009,	Cosmin Prund
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//*************************************************************************************

#include "prepconst.h"
#include "avr/interrupt.h"
#include "fastusart.h"
#include "cmdhandle.h"
#include "scopecounter.h"
#include "interactiveprompt.h"
#include "scopeadc.h"
#include "adcdump.h"

// Prepare the USART for use
void PrepUSART()
{
	// Make the UBRR==1 => 500,000 baud rate
	UBRR_H = 0;
	UBRR_L = 1;
	// Reset all status bits
	UCSR_A = 0;
	// Enable both transmitter and receiver
	UCSR_B = 0b00011000;
	// Set frame format: Asynchron, 8data, 1stop bit
	UCSR_C = 0b00000110;
}

// Program Loop
int main()
{
	PrepUSART();
	PrepareCOUNTER1();
	sei();
	ScopeOutputMode = 2;
	PrepADC();	

	while(1)
	{
		switch(ScopeOutputMode)
		{
			case 1:
				InteractiveCmdPromptLoop();
				break;
			case 2:
				DecimalTextDump_8bit_AllChannels();
				break;
			case 3:
				DecimalTextDump_10bit_AllChannels();
				break;
			case 4:
				DecimalTextDump_8bit_OneChannel();
				break;
			case 5:
				DecimalTextDump_10bit_OneChannel();
				break;
			case 6:
				BinaryDump_8bit_OneChannel();
				break;
			default:
				InteractiveCmdPromptLoop();
				break;
		}
	}

}
