#ifndef __PREPCONST__
#define __PREPCONST__

// Include local customisations
#include "customize.h"

// Include the AVR lib so I can access many-many interesting things
#include <avr/io.h>
#include <util/delay.h>

// What USART's are available? What do I select for an USART?
#ifdef UDR0
	//	Most likely multiple USARTs are available
	#ifndef USEUSART
	#ifndef __prepconst_error_
	#define __prepconst_error_
	#error "The USEUSART define in customize.h is missing. Please make sure the USEUSART is defined and is the number of the USART you want to use"
	#endif
	#endif

	// Test each USART one by one; I'm going throw this trouble because I want an pretty error
	// message if the user configured the wrong USART
	#if USEUSART == 0		
		// Prepare UBRR_H
		#ifdef UBRR0H
		#define UBRR_H UBRR0H
		#endif
		// Prepare UBRR_L
		#ifdef UBRR0L
		#define UBRR_L UBRR0L
		#endif
		// Prepare UCSR_A		
		#ifdef UCSR0A
		#define UCSR_A UCSR0A
		#endif
		// Prepare UCSR_B
		#ifdef UCSR0B
		#define UCSR_B UCSR0B
		#endif
		// Prepare UCSR_C	
		#ifdef UCSR0C
		#define UCSR_C UCSR0C
		#endif
		// Prepare UDR_
		#ifdef UDR0
		#define UDR_ UDR0
		#endif
	#endif

	#if USEUSART == 1
		// Prepare UBRR_H
		#ifdef UBRR1H
		#define UBRR_H UBRR1H
		#endif
		// Prepare UBRR_L
		#ifdef UBRR1L
		#define UBRR_L UBRR1L
		#endif
		// Prepare UCSR_A		
		#ifdef UCSR1A
		#define UCSR_A UCSR1A
		#endif
		// Prepare UCSR_B
		#ifdef UCSR1B
		#define UCSR_B UCSR1B
		#endif
		// Prepare UCSR_C	
		#ifdef UCSR1C
		#define UCSR_C UCSR1C
		#endif
		// Prepare UDR_
		#ifdef UDR1
		#define UDR_ UDR1
		#endif
	#endif

	#if USEUSART == 2
		// Prepare UBRR_H
		#ifdef UBRR2H
		#define UBRR_H UBRR2H
		#endif
		// Prepare UBRR_L
		#ifdef UBRR2L
		#define UBRR_L UBRR2L
		#endif
		// Prepare UCSR_A		
		#ifdef UCSR2A
		#define UCSR_A UCSR2A
		#endif
		// Prepare UCSR_B
		#ifdef UCSR2B
		#define UCSR_B UCSR2B
		#endif
		// Prepare UCSR_C	
		#ifdef UCSR2C
		#define UCSR_C UCSR2C
		#endif
		// Prepare UDR_
		#ifdef UDR2
		#define UDR_ UDR2
		#endif
	#endif

	#if USEUSART == 3
		// Prepare UBRR_H
		#ifdef UBRR3H
		#define UBRR_H UBRR3H
		#endif
		// Prepare UBRR_L
		#ifdef UBRR3L
		#define UBRR_L UBRR3L
		#endif
		// Prepare UCSR_A		
		#ifdef UCSR3A
		#define UCSR_A UCSR3A
		#endif
		// Prepare UCSR_B
		#ifdef UCSR3B
		#define UCSR_B UCSR3B
		#endif
		// Prepare UCSR_C	
		#ifdef UCSR3C
		#define UCSR_C UCSR3C
		#endif
		// Prepare UDR_
		#ifdef UDR3
		#define UDR_ UDR3
		#endif
	#endif

#else
	//	Most likely only one USART is available. Or NO USART is available!
	#define "USART0 is NOT available! This configuration file is not ready to handle this case."
#endif

#ifndef UDR_
	#ifndef __prepconst_error_
	#define __prepconst_error_
	#error "The USART port is not properly configured because the UDR_ simbol is not defined. Please verify the USEUSART const in customize.h and make sure the USART port is available for your MCU!"
	#endif
#endif

#ifndef UBRR_H
	#ifndef __prepconst_error_
	#define __prepconst_error_
	#error "The USART port configured in customize.h seems to be defined for your MCU but there apears to be a change in register names that this configuration file is not prepared for. Please file an bug report specifying your MCU number"
	#endif
#endif

#ifndef UBRR_L
	#ifndef __prepconst_error_
	#define __prepconst_error_
	#error "The USART port configured in customize.h seems to be defined for your MCU but there apears to be a change in register names that this configuration file is not prepared for. Please file an bug report specifying your MCU number"
	#endif
#endif

#ifndef UCSR_A
	#ifndef __prepconst_error_
	#define __prepconst_error_
	#error "The USART port configured in customize.h seems to be defined for your MCU but there apears to be a change in register names that this configuration file is not prepared for. Please file an bug report specifying your MCU number"
	#endif
#endif

#ifndef UCSR_B
	#ifndef __prepconst_error_
	#define __prepconst_error_
	#error "The USART port configured in customize.h seems to be defined for your MCU but there apears to be a change in register names that this configuration file is not prepared for. Please file an bug report specifying your MCU number"
	#endif
#endif

#ifndef UCSR_C
	#ifndef __prepconst_error_
	#define __prepconst_error_
	#error "The USART port configured in customize.h seems to be defined for your MCU but there apears to be a change in register names that this configuration file is not prepared for. Please file an bug report specifying your MCU number"
	#endif
#endif

// Prepare macros to test if an char is available (receive), send an char and test if the send/receive
// register is empty

#define USART_SEND_BYTE(c)	UDR_=(c)
#define USART_DATA_REGISTER_NOT_EMPTY bit_is_clear(UCSR_A,5)
#define USART_READ_BYTE		UDR_
#define USART_IS_RX_BYTE_AVAILABLE	bit_is_set(UCSR_A, 7)

// Try to extract device name from whatever's defined
#if defined (__AVR_AT94K__)
#define MCUNAME "AVR_AT94K"
#elif defined (__AVR_AT43USB320__)
#define MCUNAME "AVR_AT43USB320"
#elif defined (__AVR_AT43USB355__)
#define MCUNAME "AVR_AT43USB355"
#elif defined (__AVR_AT76C711__)
#define MCUNAME "AVR_AT76C711"
#elif defined (__AVR_AT86RF401__)
#define MCUNAME "AVR_AT86RF401"
#elif defined (__AVR_AT90PWM1__)
#define MCUNAME "AVR_AT90PWM1"
#elif defined (__AVR_AT90PWM2__)
#define MCUNAME "AVR_AT90PWM2"
#elif defined (__AVR_AT90PWM2B__)
#define MCUNAME "AVR_AT90PWM2B"
#elif defined (__AVR_AT90PWM3__)
#define MCUNAME "AVR_AT90PWM3"
#elif defined (__AVR_AT90PWM3B__)
#define MCUNAME "AVR_AT90PWM3B"
#elif defined (__AVR_AT90PWM216__)
#define MCUNAME "AVR_AT90PWM216"
#elif defined (__AVR_AT90PWM316__)
#define MCUNAME "AVR_AT90PWM316"
#elif defined (__AVR_ATmega32C1__)
#define MCUNAME "AVR_ATmega32C1"
#elif defined (__AVR_ATmega32M1__)
#define MCUNAME "AVR_ATmega32M1"
#elif defined (__AVR_ATmega32U4__)
#define MCUNAME "AVR_ATmega32U4"
#elif defined (__AVR_ATmega32U6__)
#define MCUNAME "AVR_ATmega32U6"
#elif defined (__AVR_ATmega128__)
#define MCUNAME "AVR_ATmega128"
#elif defined (__AVR_ATmega1280__)
#define MCUNAME "AVR_ATmega1280"
#elif defined (__AVR_ATmega1281__)
#define MCUNAME "AVR_ATmega1281"
#elif defined (__AVR_ATmega1284P__)
#define MCUNAME "AVR_ATmega1284P"
#elif defined (__AVR_ATmega2560__)
#define MCUNAME "AVR_ATmega2560"
#elif defined (__AVR_ATmega2561__)
#define MCUNAME "AVR_ATmega2561"
#elif defined (__AVR_AT90CAN32__)
#define MCUNAME "AVR_AT90CAN32"
#elif defined (__AVR_AT90CAN64__)
#define MCUNAME "AVR_AT90CAN64"
#elif defined (__AVR_AT90CAN128__)
#define MCUNAME "AVR_AT90CAN128"
#elif defined (__AVR_AT90USB82__)
#define MCUNAME "AVR_AT90USB82"
#elif defined (__AVR_AT90USB162__)
#define MCUNAME "AVR_AT90USB162"
#elif defined (__AVR_AT90USB646__)
#define MCUNAME "AVR_AT90USB646"
#elif defined (__AVR_AT90USB647__)
#define MCUNAME "AVR_AT90USB647"
#elif defined (__AVR_AT90USB1286__)
#define MCUNAME "AVR_AT90USB1286"
#elif defined (__AVR_AT90USB1287__)
#define MCUNAME "AVR_AT90USB1287"
#elif defined (__AVR_ATmega64__)
#define MCUNAME "AVR_ATmega64"
#elif defined (__AVR_ATmega640__)
#define MCUNAME "AVR_ATmega640"
#elif defined (__AVR_ATmega644__)
#define MCUNAME "AVR_ATmega644"
#elif defined (__AVR_ATmega644P__)
#define MCUNAME "AVR_ATmega644P"
#elif defined (__AVR_ATmega645__)
#define MCUNAME "AVR_ATmega645"
#elif defined (__AVR_ATmega6450__)
#define MCUNAME "AVR_ATmega6450"
#elif defined (__AVR_ATmega649__)
#define MCUNAME "AVR_ATmega649"
#elif defined (__AVR_ATmega6490__)
#define MCUNAME "AVR_ATmega6490"
#elif defined (__AVR_ATmega103__)
#define MCUNAME "AVR_ATmega103"
#elif defined (__AVR_ATmega32__)
#define MCUNAME "AVR_ATmega32"
#elif defined (__AVR_ATmega323__)
#define MCUNAME "AVR_ATmega323"
#elif defined (__AVR_ATmega324P__)
#define MCUNAME "AVR_ATmega324P"
#elif defined (__AVR_ATmega325__)
#define MCUNAME "AVR_ATmega325"
#elif defined (__AVR_ATmega325P__)
#define MCUNAME "AVR_ATmega325P"
#elif defined (__AVR_ATmega3250__)
#define MCUNAME "AVR_ATmega3250"
#elif defined (__AVR_ATmega3250P__)
#define MCUNAME "AVR_ATmega3250P"
#elif defined (__AVR_ATmega328P__)
#define MCUNAME "AVR_ATmega328P"
#elif defined (__AVR_ATmega329__)
#define MCUNAME "AVR_ATmega329"
#elif defined (__AVR_ATmega329P__)
#define MCUNAME "AVR_ATmega329P"
#elif defined (__AVR_ATmega3290__)
#define MCUNAME "AVR_ATmega3290"
#elif defined (__AVR_ATmega3290P__)
#define MCUNAME "AVR_ATmega3290P"
#elif defined (__AVR_ATmega32HVB__)
#define MCUNAME "AVR_ATmega32HVB"
#elif defined (__AVR_ATmega406__)
#define MCUNAME "AVR_ATmega406"
#elif defined (__AVR_ATmega16__)
#define MCUNAME "AVR_ATmega16"
#elif defined (__AVR_ATmega161__)
#define MCUNAME "AVR_ATmega161"
#elif defined (__AVR_ATmega162__)
#define MCUNAME "AVR_ATmega162"
#elif defined (__AVR_ATmega163__)
#define MCUNAME "AVR_ATmega163"
#elif defined (__AVR_ATmega164P__)
#define MCUNAME "AVR_ATmega164P"
#elif defined (__AVR_ATmega165__)
#define MCUNAME "AVR_ATmega165"
#elif defined (__AVR_ATmega165P__)
#define MCUNAME "AVR_ATmega165P"
#elif defined (__AVR_ATmega168__)
#define MCUNAME "AVR_ATmega168"
#elif defined (__AVR_ATmega168P__)
#define MCUNAME "AVR_ATmega168P"
#elif defined (__AVR_ATmega169__)
#define MCUNAME "AVR_ATmega169"
#elif defined (__AVR_ATmega169P__)
#define MCUNAME "AVR_ATmega169P"
#elif defined (__AVR_ATmega8HVA__)
#define MCUNAME "AVR_ATmega8HVA"
#elif defined (__AVR_ATmega16HVA__)
#define MCUNAME "AVR_ATmega16HVA"
#elif defined (__AVR_ATmega8__)
#define MCUNAME "AVR_ATmega8"
#elif defined (__AVR_ATmega48__)
#define MCUNAME "AVR_ATmega48"
#elif defined (__AVR_ATmega48P__)
#define MCUNAME "AVR_ATmega48P"
#elif defined (__AVR_ATmega88__)
#define MCUNAME "AVR_ATmega88"
#elif defined (__AVR_ATmega88P__)
#define MCUNAME "AVR_ATmega88P"
#elif defined (__AVR_ATmega8515__)
#define MCUNAME "AVR_ATmega8515"
#elif defined (__AVR_ATmega8535__)
#define MCUNAME "AVR_ATmega8535"
#elif defined (__AVR_AT90S8535__)
#define MCUNAME "AVR_AT90S8535"
#elif defined (__AVR_AT90C8534__)
#define MCUNAME "AVR_AT90C8534"
#elif defined (__AVR_AT90S8515__)
#define MCUNAME "AVR_AT90S8515"
#elif defined (__AVR_AT90S4434__)
#define MCUNAME "AVR_AT90S4434"
#elif defined (__AVR_AT90S4433__)
#define MCUNAME "AVR_AT90S4433"
#elif defined (__AVR_AT90S4414__)
#define MCUNAME "AVR_AT90S4414"
#elif defined (__AVR_ATtiny22__)
#define MCUNAME "AVR_ATtiny22"
#elif defined (__AVR_ATtiny26__)
#define MCUNAME "AVR_ATtiny26"
#elif defined (__AVR_AT90S2343__)
#define MCUNAME "AVR_AT90S2343"
#elif defined (__AVR_AT90S2333__)
#define MCUNAME "AVR_AT90S2333"
#elif defined (__AVR_AT90S2323__)
#define MCUNAME "AVR_AT90S2323"
#elif defined (__AVR_AT90S2313__)
#define MCUNAME "AVR_AT90S2313"
#elif defined (__AVR_ATtiny2313__)
#define MCUNAME "AVR_ATtiny2313"
#elif defined (__AVR_ATtiny13__)
#define MCUNAME "AVR_ATtiny13"
#elif defined (__AVR_ATtiny13A__)
#define MCUNAME "AVR_ATtiny13A"
#elif defined (__AVR_ATtiny25__)
#define MCUNAME "AVR_ATtiny25"
#elif defined (__AVR_ATtiny45__)
#define MCUNAME "AVR_ATtiny45"
#elif defined (__AVR_ATtiny85__)
#define MCUNAME "AVR_ATtiny85"
#elif defined (__AVR_ATtiny24__)
#define MCUNAME "AVR_ATtiny24"
#elif defined (__AVR_ATtiny44__)
#define MCUNAME "AVR_ATtiny44"
#elif defined (__AVR_ATtiny84__)
#define MCUNAME "AVR_ATtiny84"
#elif defined (__AVR_ATtiny261__)
#define MCUNAME "AVR_ATtiny261"
#elif defined (__AVR_ATtiny461__)
#define MCUNAME "AVR_ATtiny461"
#elif defined (__AVR_ATtiny861__)
#define MCUNAME "AVR_ATtiny861"
#elif defined (__AVR_ATtiny43U__)
#define MCUNAME "AVR_ATtiny43U"
#elif defined (__AVR_ATtiny48__)
#define MCUNAME "AVR_ATtiny48"
#elif defined (__AVR_ATtiny88__)
#define MCUNAME "AVR_ATtiny88"
#elif defined (__AVR_ATtiny167__)
#define MCUNAME "AVR_ATtiny167"
#endif

#ifndef MCUNAME
#define MCUNAME "Unkown device"
#endif

#endif
