//*************************************************************************************
//
//  SoR Scope - software to provide Oscilloscope-like with cheap Microcontrollers
//  and free PC software
//
//  Copyright (C) 2009,	Cosmin Prund
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//*************************************************************************************

#include "scopeadc.h"
#include "prepconst.h"
#include "customize.h"

unsigned char AvailableAdcCount;
char * AdcNames[MAX_NUM_ADC];
char AdcBitMask[MAX_NUM_ADC];
volatile uint8_t * AdcDDRRegister[MAX_NUM_ADC];
unsigned char AdcMuxMask[MAX_NUM_ADC];

void ConfigADCPin(char * PinName, char PinNum, volatile uint8_t * DDRReg, char MuxMask)
{
	if (AvailableAdcCount < MAX_NUM_ADC) {
		AdcNames[AvailableAdcCount] = PinName;
		AdcBitMask[AvailableAdcCount] = 1 << PinNum;
		AdcDDRRegister[AvailableAdcCount] = DDRReg;
		// Next
		AvailableAdcCount++;
	}
}

// Prepare voltage reference
void PrepVoltageRef()
{
	// Some day this will be configurable
	#ifdef REFS1
	ADMUX = (ADMUX & ~((1 << REFS1) | (1 << REFS0))) | (1 << REFS0);
	#else
	ADMUX &= ~(1 << REFS0);
	#endif
}

void AdcLeftAdjustResult()
{
	#ifdef ADLAR
	ADMUX |= (1 << ADLAR);
	#endif
}

void AdcRightAdjustResult()
{
	#ifdef ADLAR
	ADMUX &= ~(1 << ADLAR);
	#endif
}

// Prepare ADC prescaler clock
void PrepPrescalerClock()
{
	// This also needs to be configurable! For now I'll just set the prescaler to NOTHING (000 - minimum delay)
	// because that's enough for our purposes
	#ifdef ADPS2
	ADCSRA = (ADCSRA & ~(ADPS0 | ADPS1 | ADPS2)) | ADPS2;
	#else
	ADCSRA = (ADCSRA & ~(ADPS0 | ADPS1)) | ADPS1;
	#endif
}

// Prepare ADC for use
void PrepADC()
{
	// Init global stuff
	AvailableAdcCount = 0;

	// Play tricks on the "C" precompiler; Make it so I get part of this procedure from
	// "customize.h" - pretty usefull!
	#define __PREPCONST_ADC_PINS_
	#include "customize.h"
	#undef __PREPCONST_ADC_PINS_

	// Prepare the configuration for the usual stuff
	ADCSRA=0;
	ADCSRB=0;
	ADMUX=0;

	// Mark every single analog pin as disabled for digital input in it's correspnding DDR register
	for (int i=0;i<AvailableAdcCount;i++)
		*AdcDDRRegister[i] |= AdcBitMask[i];

	// Prepare the voltage reference
	PrepPrescalerClock();
	AdcLeftAdjustResult();
	PrepVoltageRef();

}
